*! version 0.3.8 2014-08-16 | scott long | _help
*  version 0.3.7 2014-08-16 | scott long | table
*  version 0.3.6 2014-08-10 | scott long | diag
*  version 0.3.5 2014-08-09 | scott long | add get transpose
*  version 0.3.4 2014-08-08 | scott long | format cleanup
*  version 0.3.3 2014-08-08 | scott long | preserve returns
*  version 0.3.2 2014-08-08 | scott long | allow rows(l) cols(l)
*  version 0.3.1 2014-08-08 | scott long | abs, trap dimenssions
*  version 0.3.0 2014-08-02 | scott long | tempname
*  version 0.2.1 2014-08-01 | scott long | scalar

capture program drop matmath
capture program drop _help

program _help
di
di " matmath newmat = L operator R"
di " "
di "     math : + - * / ^"
di "     %of  : L as percent of R{"
di "     %dif : change from L to R as % of L"
di "     \ ,  :  row or column concantenate (,, \\ with mat_*app)"
di " "
di " matmath newmat = operator(old)"
di " "
di "     abs sqrt q log exp inv log10 odds logit"
di " "
di " matmath newmat = diag(old) : vector from diagonal"
di " matmath newmat = keep(old), rows(indices) | cols(indices)"
di " matmath newmat = drop(old), rows(indices) | cols(indices)"
di " matmath newmat =  add(old), row(index) col(index) value(#) name()"
di " "
di "     indices: #, p-rior, l-ast, n-ext"
di " "
di " matmath newmat = get(mat-name)"
di " matmath newmat = e(return-name) : retrieve e-return matrix"
di " matmath newmat = r(return-name) : retrieve r-return matrix"
di " matmath newmat = table | eb | ev | rb | rv"
di
di " options"
di
di "     list, transpose"
end

program matmath, sclass

    version 11.2

    tempname origreturns
    _return hold `origreturns'
    _return restore `origreturns', hold

    local binaryops "+ - / * ^ , ,, \ \\ %of %dif"
    local unaryops "sqrt sq log log10 exp inv power abs"
    local getops "get r e diag"
    local getreturnops "table eb ev rb rv"
    local bookops "keep drop add"

    local input "`*'"

    if "`input'"=="" {
        _help
        exit
    }

    * make cols() a synonym for columns()
    local input : subinstr local input "cols(" "col(", all

//  get options

    * remove , so _decodeoptions can use syntax
    local tempinput : subinstr local input "," " ", all

//  determine operation being requested

    * ,, to ! and \\ to @ to distinguish ,, from ,; \\ from \
    local input : subinstr local input ",," "!", all
    local input : subinstr local input "\\" "@", all

    foreach t in = `binaryops' {
        local input = subinstr("`input'","`t'"," `t' ",.) // space around operators
    }
    local input = subinstr("`input'","  "," ",.)
    local input : subinstr local input "!" " ,, ", all
    local input : subinstr local input "@" " \\ ", all
    local input : subinstr local input " (" "(", all
    local input : subinstr local input "(" "( ", all
    local input : subinstr local input ")" " )", all

    * isbook
    local isbook = 0
    foreach t in `bookops' {
        local ipos : list posof "`t'(" in input
        if `ipos'>0 {
            local isbook = 1
            local is`t' = 1
            local operator "`t'"
        }
    }

    * isget
    local isget = 0
    foreach t in `getops' {
        local ipos : list posof "`t'(" in input
        if `ipos'>0 {
            local isget = 1
            local is`t' = 1
            local operator "`t'"
        }
    }

    * isgetreturn
    local isgetreturn = 0
    foreach t in `getreturnops' {
        local ipos : list posof "`t'" in input
        if `ipos'>0 {
            local isgetreturn = 1
            local is`t' = 1
            local operator "`t'"
        }
    }

    * isunary
    local isunary = 0
    foreach t in `unaryops' {
        local ipos : list posof "`t'(" in input
        if `ipos'>0 {
            local isunary = 1
            local is`t' = 1
            local operator "`t'"
        }
    }

    * isbinary
    local isbinary = 0
    if !`isbook' & !`isunary' & !`isget' & !`isgetreturn' {
        local isbinary = 1
    }

//  matrices

    if `isbinary' {
        _decodebinary `input'
        local matout `s(matout)'
        local eq "`s(eq)'"
        local matleft `s(matleft)'
        local matright `s(matright)'
        local operator `s(op)'
        local ipos : list posof "`operator'" in binaryops
        if `ipos'==0 {
            di as error "operator `operator' is invalid"
            exit
        }
    }

    if `isunary' | `isbook' | `isget' | `isgetreturn' {
        local matout : word 1 of `input'
        local eq : word 2 of `input'
        local matleft : word 4 of `input'
        local matright ""
        local isscalar = 0
    }

    local iserror = 0

    if `isgetreturn' {
        local matleft `operator'
        if ("`operator'"=="eb" | "`operator'"=="rb") local matleft b
        if ("`operator'"=="ev" | "`operator'"=="rv") local matleft V
        local isereturn = 0
        local isrreturn = 0
        if ("`operator'"=="eb" | "`operator'"=="ev") local isereturn = 1
        else local isrreturn = 1

        local isget = 1 // with new matleft, treat just like get
    }

    if `isget' {
        tempname ermat
        if "`operator'"=="e" | `isereturn' {
            capture confirm matrix e(`matleft')
            if _rc {
                di as error "matrix e(`matleft') does not exist"
                local iserror = 1
            }
            else {
                matrix `ermat' = e(`matleft')
                local matleft "`ermat'"
            }
        }
        else if "`operator'"=="r" | `isrreturn'  {
            capture confirm matrix r(`matleft')
            if _rc {
                di as error "matrix r(`matleft') does not exist"
                local iserror = 1
            }
            else {
                matrix `ermat' = r(`matleft')
                local matleft "`ermat'"
            }
        }
        else if "`operator'"=="diag" {
            matrix `ermat' = vecdiag(`matleft')
            local origmatleft "`matleft'"
            local matleft "`ermat'"
        }
    }

    if "`eq'"!="=" {
        di as error "missing = sign"
        local iserror = 1
    }

    capture confirm matrix `matleft'
    if _rc {
        di as error "matrix `matleft' does not exist"
        local iserror = 1
    }
    else { // needed for col(l) row(l)
        local matleftrowsN = rowsof(`matleft')
        local matleftcolsN = colsof(`matleft')
}

    if `isbinary' {

        tempname tmp
        * is matrix?
        capture confirm matrix `matright'
        if _rc local ismatrix = 0 // not matrix
        else local ismatrix = 1

        * is scalar?
        local isscalar = 0
        if `ismatrix'==0 { // is it a scalar
            capture scalar `tmp' = `matright'
            capture confirm scalar `tmp'
            if _rc local isscalar = 0
            else local isscalar = 1
        }
        if `isscalar'==0 & `ismatrix'==0 {
            local iserror = 1
            di as error ///
            "the right element in `matleft' `operator' `matright' does not exist"
        }
        if !`isscalar' { // two matrices being used
            * local matleftrowsN = rowsof(`matleft') // computed above
            * local matleftcolsN = colsof(`matleft')
            local matrightrowsN = rowsof(`matright')
            local matrightcolsN = colsof(`matright')
            if "`operator'"=="," {
                if `matleftrowsN'!=`matrightrowsN' {
                    display as error ///
                    "dimensions of matrices are incompatabile"
                    local iserror = 1
                }
            }
            else if "`operator'"=="\" {
                if `matleftcolsN'!=`matrightcolsN' {
                    display as error ///
                    "dimensions of matrices are incompatabile"
                    local iserror = 1
                }
            }
            else if "`operator'"!=",," & "`operator'"!="\\" {
                if (`matleftrowsN'!=`matrightrowsN' ///
                    | `matleftcolsN'!=`matrightcolsN')  {
                    display as error ///
                    "dimensions of matrices are incompatabile"
                    local iserror = 1
                }
            }

        } // not scalar for second element
        if `iserror'==1 exit

    } // binary

    _decodeoptions, ///
        `tempinput' rowlast(`matleftrowsN') collast(`matleftcolsN') ///
        value(string)
    local addvalue "`s(value)'"
    local name `"`s(name)'"'
    local list "`s(list)'"
    local colis "`s(columns)'"
    local rowis "`s(rows)'"
    local format "`s(format)'"
    local transpose "`s(transpose)'"

    if "`operator'"=="diag" {
        if "`name'"=="" local name "diag_`origmatleft'"
    }

    * decode add options
    if "`operator'"=="add" {
        if "`rowis'"=="" & "`colis'"=="" {
            display as error ///
            "row() and/or col() is required with add"
            local iserror = 1
        }
        if "`rowis'"!="" {
            local rowis : word 1 of `rowis' // only one row allowed
            if `rowis'<0 | `rowis'>(`matleftrowsN'+1) {
                display as error ///
                "row() value must be between 0 and `matleftrowsN'"
                local iserror = 1
            }
        }
        if "`colis'"!="" {
            local colis : word 1 of `colis' // only one col allowed
            if `colis'<0 | `colis'>(`matleftcolsN'+1) {
                display as error ///
                "col() value must be between 0 and `matleftcolsN'"
                local iserror = 1
            }
        }
    } // add

/*
di "input is |`input'|"
di "DECODED"
di "isunary = `isunary'     operator!`operator'!"
di "isbook = `isbook'       operator!`operator'!"
di "isbinary = `isbinary'   operator!`operator'!"
di "matout: `matout'    matleft:   `matleft'      matright:   `matright'"
di "list!`list'!        colis !`colis'!     rowis !`rowis'!"
di "rowis: `rowis'   colis: `colis'   value: `addvalue'"
*/
    if `iserror' {
        exit
        _return restore `origreturns'
    }

//  dimensions

    _matmath , matout(`matout') matleft(`matleft') matright(`matright') ///
        isunary(`isunary') isbinary(`isbinary') isbook(`isbook') ///
        isget(`isget') isscalar(`isscalar') operator(`operator') ///
        `list' cols(`colis') rows(`rowis') format(`format') ///
        addvalue(`addvalue') name(`name') ///
        `transpose'

    _return restore `origreturns'
    local nrows = rowsof(`matout')
    local ncols = colsof(`matout')
    sreturn local nrows `nrows'
    sreturn local ncols `ncols'
end

program _decodeoptions, sclass
    syntax [ ,  Rows(string) Columns(string) list ///
        rowlast(string) collast(string) ///
        format(string) Value(string) name(string) ///
        transpose Width(integer 9) Decimals(integer 3) ///
        * ]

    if ("`value'"!="") local value = real("`value'")
    else local value = real(".") // default

    if "`rows'"!="" {
        * l is turned to last row number
        local rows : subinstr local rows "l" "`rowlast'", all
        numlist "`rows'"
        local rows "`r(numlist)'"
    }
    if "`columns'"!="" {
        local columns : subinstr local columns "l" "`collast'", all
        numlist "`columns'"
        local columns "`r(numlist)'"
    }

    if ("`format'"=="") local format "%`width'.`decimals'f"

    sreturn local name `name'
    sreturn local value `value'
    sreturn local rows `rows'
    sreturn local columns `columns'
    sreturn local list `list'
    sreturn local format `format'
    sreturn local transpose `transpose'

end

program _decodebinary, sclass
    args matout eq matleft op matright opt1 opt2 opt3 opt4
    sreturn local matout `matout'
    sreturn local eq "`eq'"
    sreturn local matleft `matleft'
    sreturn local op "`op'"
    sreturn local matright `matright'
end

exit
